@@ -16,6 +16,7 @@ def interaction_matrix(N,K,shape="roll"):
     Returns:
         numpy.ndarray: an NxN numpy array with diagonal values equal to 1 and rowSums=colSums=K+1
     """
+
     output = None
     if K == 0:
         output = np.eye(N,dtype=int)
@@ -58,6 +59,7 @@ def binary_combinations(N,R):
     Returns:
         numpy.ndarray: an (N R)xN numpy array, where rows are vectors of size N and sum of elements R
     """
+
     tmp = []
     idx = comb(range(N),R)
     for i in idx:
@@ -79,6 +81,7 @@ def random_binary_matrix(N,R,diag=None):
     Returns:
         numpy.ndarray: an NxN numpy array
     """
+
     if N==R:
         tmp = np.ones((N,R),dtype=int)
         return tmp
@@ -139,16 +142,14 @@ def binx(x,size=4,out=None):
         list: A list of 0s and 1s if x is int.
         int: A decimal equivalent if x is str, numpy.ndarray, list.
     """
+
     tmp = x
     if type(tmp) is int:
         tmp = np.binary_repr(tmp,size)
         tmp = [int(z) for z in tmp]
     elif type(tmp) is str:
         tmp = int(tmp,2)
-    elif type(tmp) is np.ndarray:
-        lenx = len(tmp)
-        tmp = np.sum(np.flip(tmp) * 2 ** (np.arange(len(tmp))))
-    elif type(tmp) is list:
+    elif type(tmp) is np.ndarray or type(tmp) is list:
         tmp = np.sum(np.flip(tmp) * 2 ** (np.arange(len(tmp))))
     else:
         print("incorrect input for function binx")
@@ -168,6 +169,7 @@ def contrib_define(p,n,k,c,s,rho):
     Returns:
         numpy.ndarray: An (N*P)x2**(1+K+C*S) matrix of P contribution matrices with correlation RHO
     """
+
     #output = np.random.uniform(0,1,(2**(1+k+c*s),n*p))
     corrmat = np.repeat(rho,p*p).reshape(p,p) + (1-rho) * np.eye(p)
     corrmat = 2*np.sin((np.pi / 6 ) * corrmat)
@@ -178,35 +180,6 @@ def contrib_define(p,n,k,c,s,rho):
     return output
 
 @njit
-def xcontrib_solve(x,imat,cmat,n,p):
-    n_p = n*p
-    phi = [0.0]*n_p
-    for i in range(n_p):
-        tmq = np.arange(n_p)#imat[:,i].copy()
-        q = int(i/n)
-        m = i % n
-        tmq = np.concatenate((tmq[q*n:(q+1)*n],tmq[0:q*n],tmq[(q+1)*n:n_p]))
-        tmq = np.concatenate((tmq[m:n],tmq[0:m],tmq[n:n_p]))
-        tmx = np.where(imat[:,i]>0)[0]
-        tmpx = []
-        for j in tmq:
-            a = False
-            for jj in tmx:
-                a = a or (j == jj)
-            tmpx.append(a)
-        tmpx = np.array(tmpx)
-        # used loops, because jit does not support list comprehensions
-        #tmpx = [z in tmx for z in tmq]
-        #tmpx = np.isin(tmq,tmx)
-        #tmpx = np.in1d(tmq,tmx)
-
-        indices = tmq[tmpx]
-
-        tmp = x[indices]
-        tmp_loc = np.sum(np.flip(tmp) * 2 ** (np.arange(len(tmp)))) #binx(tmp)
-        phi[i] = cmat[tmp_loc,i]
-    return phi
-
 def contrib_solve(x,imat,cmat,n,p):
     """Computes a performance for vector x from given contribution matrix and interaction matrix
 
@@ -223,9 +196,18 @@ def contrib_solve(x,imat,cmat,n,p):
     Returns:
         float: A mean performance of an input vector x given cmat and imat.
     """
-    phi = xcontrib_solve(x,imat,cmat,n,p)
-    phi = np.reshape(phi,(p,n)).mean(axis=1)
-    return phi
+    
+    n_p = n*p
+    phi = np.zeros(n_p)
+    for i in range(n_p):
+        tmp = x[np.where(imat[:,i]>0)] # coupled bits
+        tmp_loc = np.sum(np.flip(tmp) * 2 ** (np.arange(len(tmp)))) # convert to integer
+        phi[i] = cmat[tmp_loc,i]
+
+    output = [0.0]*p
+    for i in range(p):
+        output[i] = phi[i*n : (i+1)*n].mean()
+    return output
 
 def contrib_full(imat,cmat,n,p):
     """Computes performances for all binary vectors of size N*P
@@ -243,8 +225,9 @@ def contrib_full(imat,cmat,n,p):
         numpy.ndarray: Array of performances for all vectors of size N*P, normalized by the global maximum
         float: the global maximum value
     """
+
     n_p = n*p
-    perfmat = np.zeros((2**n_p,p),dtype=float)
+    perfmat = np.empty((2**n_p,p),dtype=float)
     for i in range(2**n_p):
         bstring = np.array(binx(i,n_p))
         bval = contrib_solve(bstring,imat,cmat,n,p) # !!! processing heavy !!!
@@ -261,18 +244,19 @@ def contrib_full(imat,cmat,n,p):
     output3 = perfmax
     return output1, output3#, output2, output3
 
-#def getglobalmax2(imat,cmat,n,p):
-#    n_p = n*p
-#    perfmax = np.zeros(p,dtype=float)
-#    perfargmax = 0
-#    for i in range(2**n_p):
-#        print(i)
-#        bstring = np.array(binx(i,n_p))
-#        bval = contrib_solve(bstring,imat,cmat,n,p) # !!! processing heavy !!!
-#        perfmax = np.maximum(perfmax, bval)
-#
-#    output = perfmax
-#    return output
+def get_globalmax_brute(imat,cmat,n,p):
+    n_p = n*p
+    perfargmax = 0
+    bstrings = map(lambda x: np.array(binx(x,n_p)), range(2**n_p))
+
+    perfmax = [0.0]*p#np.zeros(p,dtype=float)
+    for i in bstrings:#range(2**n_p):
+        bstring = i
+        bval = contrib_solve(bstring,imat,cmat,n,p) # !!! processing heavy !!!
+        perfmax = np.maximum(perfmax, bval)
+
+    output = np.array(perfmax)
+    return output
 
 def get_globalmax(imat,cmat,n,p,t0=20,t1=0,alpha=0.1,kk=1):
     """Estimates a global maximum for a landscape using Simulated Annealing algorithm
@@ -290,6 +274,7 @@ def get_globalmax(imat,cmat,n,p,t0=20,t1=0,alpha=0.1,kk=1):
     Returns:
         numpy.ndarray: The estimates of the global maximum for each of the P landscapes
     """
+
     n_p = n*p
     
     state = np.array(binx(0,n_p))
@@ -324,6 +309,7 @@ def assign_tasks(N,POP,shape="solo"):
     Returns:
         numpy.ndarray: Returns a POPxN matrix where rows represent agents and cols represent tasks
     """
+
     output = None
     if shape=="solo":
         perCapita = N / POP
@@ -337,41 +323,38 @@ def assign_tasks(N,POP,shape="solo"):
 
 ###############################################################################
 
-def generate_network(POP,S=[2],shape="random",absval=False,weights=[1.0]):
+def generate_network(POP,S=2,pcom=1.0,shape="random",absval=False):
     """Generates a unidirectional network topology
 
     Args:
         POP (int): Number of agents (population size)
-        S (list): A list of network degrees (list is used for averaging different network structures)
+        S (int): Network degree
+        pcom (float): Probability of communicating through the channel
         shape (str): A network topology. Takes values 'random' (default), 'cycle' (ring topology), 'star' (not used at the moment)
         absval (bool): Indexing convention. If True, converts negative indices to positive.
-        weights (list): A list of weights for different network degrees
 
     Returns:
         numpy.ndarray: A POPxPOP matrix with probabilities of connecting to other agents
         list: A list of S-sized vectors for every agent, if shape is 'cycle' (to be fixed)
     """
-    if type(S) is int:
-        print("Error: wrap S in a list")
-        return 0
-    elif max(S)>=POP:
+
+    if S>=POP:
         print("Error: wrong network degree")
         return 0
+    if pcom>1 or pcom<0:
+        print("Error: wrong probability of communication")
+        return 0
     output = None
     if S == 0:
         output = []
     elif shape=="cycle":
-        tmp = [[z-1] + [_%POP for _ in range(z+1,z+S[0])] for z in range(POP)]
+        tmp = [[z-1] + [_%POP for _ in range(z+1,z+S)] for z in range(POP)]
         if absval==True:
-            tmp = [[(z-1)%POP] + [_%POP for _ in range(z+1,z+S[0])] for z in range(POP)]
+            tmp = [[(z-1)%POP] + [_%POP for _ in range(z+1,z+S)] for z in range(POP)]
         output = tmp
     elif shape == "random":
-        tmp = []
-        for ss in S:
-            tmp.append(random_binary_matrix(POP,ss,0))
-        output = np.average(tmp,0,weights)
-        #output = np.where(tmp>0)[1]
-        #output = np.array_split(output,POP)
+        tmp = random_binary_matrix(POP,S,0)
+        output = tmp * pcom
     else:
         print(f"Unrecognized network shape '{shape}'")
 
@@ -388,6 +371,7 @@ def generate_couples(POP,S=2,shape="cycle"):
     Returns:
         list: A list of S-sized vectors with couples for every landscape.
     """
+
     if S>=POP:
         print("Error: wrong network degree")
         return 0
@@ -413,6 +397,7 @@ def get_neighbours(vec,count):
         list: A list of 1-bit neighbours of vec
         list: A list of decimal equivalents of the above
     """
+
     tmpv = []
     tmpi = []
     subbset = np.random.choice(np.arange(len(vec)),count,replace=False)
@@ -434,6 +419,7 @@ def random_neighbour(vec,myid,n):
     Returns:
         list: A vector with one bit flipped for agent myid
     """
+
     tmp = vec.copy()
     rnd = np.random.choice(range(myid*n,(myid+1)*n))
     tmp[rnd] = 1- tmp[rnd]
@@ -451,6 +437,7 @@ def get_index(vec,myid,n):
     Returns:
         int: A decimal value of vec for myid
     """
+
     tmp = vec.copy()
     tmp = binx(tmp[myid*n:(myid+1)*n])
     output = tmp
@@ -466,6 +453,7 @@ def with_noise(vec,pb=0):
     Returns:
         numpy.ndarray: A vector of same size as vec but with errors
     """
+
     if vec.size==0:
         return vec
     tmp = vec
@@ -487,6 +475,7 @@ def beta_mean(x,y):
     Returns:
         float: A float that returns x/(x+y)
     """
+    
     output = x / (x+y)
     return output
 
@@ -499,6 +488,7 @@ def flatten(x):
     Returns:
         list: A flat list
     """
+    
     output = [_ for z in x for _ in z]
     return output
 
@@ -521,6 +511,7 @@ def calculate_freq(x,vec):
     Returns:
         float: Frequency of vec in x
     """
+
     if x is None or vec.size==0 or vec[0,0]==-1:
         return 0.0
     freq = np.mean(vec,axis=0)
@@ -542,6 +533,7 @@ def extract_soc(x,myid,n,nsoc):
     Returns:
         numpy.ndarray: A vector of size nsoc
     """
+
     tmp = x[(myid+1)*n-nsoc:(myid+1)*n]
     #tmp = str(tmp).replace(" ","").replace("[","").replace("]","")
     output = tmp
@@ -565,12 +557,14 @@ def pick(vec,count):
     Returns:
         numpy.ndarray: A vector of size count
     """
+
     tmp = np.random.choice(range(vec),count,replace=False)
     output = np.sort(tmp)
     return output
 
 def artify(n,p,r):
     """depreciated"""
+
     tmp = np.arange(n*p)
     tmp = tmp.reshape(p,n)
     fnc = lambda z: np.random.choice(z,r,replace=False)
@@ -580,6 +574,7 @@ def artify(n,p,r):
 
 def calculate_match(x,art):
     """depreciated"""
+
     if art==[]:
         return 0.0
     tmp = [x[z[0]]==z[1] for z in art]
@@ -598,6 +593,7 @@ def cobb_douglas(weights,vec):
     Returns:
         float: A Cobb-Douglas value
     """
+
     x = [z+1 for z in vec]
     w = weights
     tmp = np.power(x,w).prod()
@@ -615,6 +611,7 @@ def satisfice(x,y,u):
     Returns:
         float: Utility value
     """
+
     cond = (x >= u)
     tmp = cond * (y + u) + (1-cond) * x
     output = tmp
@@ -632,6 +629,7 @@ def weighted(x,y,p1,p2):
     Returns:
         float: A weighted sum
     """
+
     tmp = p1*x + p2*y
     output = tmp
     return output
@@ -649,6 +647,7 @@ def goal_prog(perf1,perf2,u,p1,p2):
     Returns:
         float: A GP output
     """
+
     d1 = np.max((u[0]-perf1,0))
     d2 = np.max((u[1]-perf2,0))
     tmp = p1*d1 + p2*d2
@@ -657,6 +656,7 @@ def goal_prog(perf1,perf2,u,p1,p2):
 
 def schism(perf1,perf2,social):
     """depreciated"""
+
     tmp = None
     if social is True:
         tmp = perf2
